﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Excel;
using Infragistics.Windows.DataPresenter;
using System.IO;



namespace xamDataGridExcelExporter
{
    public class xamDataGridExcelExporter
    {
        #region WorkSheet Variables
        private Workbook excelWorkbook;
        private Worksheet excelWorksheet;
        private int excelWorksheetCounter;
        #endregion


        public xamDataGridExcelExporter()
        {
            excelWorksheetCounter = 0;
            excelWorkbook = new Workbook();
            excelWorksheet = excelWorkbook.Worksheets.Add("XamDataGridWorkSheet1");

        }

        public void Export(XamDataGrid xamDataGrid, string  FileName)
        {
            //Send Root Records Collection to the Export Method
            ExportXamDataGridRecords(xamDataGrid.Records);

            //Save the Records to Excel
            excelWorkbook.Save(FileName);

            //Reset the counter incase export is called again
            excelWorksheetCounter = 0;

        }

        public void Export(XamDataGrid xamDataGrid, Stream FileStreamLocation)
        {
            //Send Root Records Collection to the Export Method
            ExportXamDataGridRecords(xamDataGrid.Records);

            //Save the Records to Excel
            excelWorkbook.Save(FileStreamLocation);

            //Reset the counter incase export is called again
            excelWorksheetCounter = 0;
        }

        private void ExportXamDataGridRecords(RecordCollectionBase records)
        {
            if (records.GetType() == typeof(GroupByRecordCollection))
            {
                foreach (GroupByRecord groupByRecord in records)
                {
                    //Make a WorkSheetRow our of the GroupBy Record
                    MakeWorkSheetRow(groupByRecord as Record);

                    //Check For Children
                    if (groupByRecord.HasChildren)
                    {
                        //cast as a groupByRecord
                        //GroupByRecord gbr = groupByRecord as GroupByRecord;                       
                        //Recursively call the ExportXamDataPresenterRecords method in order to do a depth first search
                        //ExportXamDataPresenterRecords(gbr.ChildRecords as RecordCollectionBase);
                        ExportXamDataGridRecords(groupByRecord.ChildRecords as RecordCollectionBase);
                    }

                }
            }
            else
            {
                foreach (Record record in records)
                {
                    //Check to see if dr is null.  I have to do this incase there is multiple levels of grouping
                    //The second level of records is considered just a records collection and not a groupbyrecords collection
                    //This means I must see if the DataRecord is null, then I know it is a GroupByRecord.
                    DataRecord dr = record as DataRecord;
                    if ( dr != null && record.Index == 0)
                    {
                        MakeWorkSheetHeaderRow(record);
                    }   
                    //make a WorksheetRow out of the DataRecord
                    MakeWorkSheetRow(record);
                    //check for children 
                    if (record.HasChildren)
                    {
                        if (dr != null)
                        {
                            //Recursively call the ExportXamDataPresenterRecords method in order to do a depth first search
                            ExportXamDataGridRecords(dr.ChildRecords[dr.ChildRecords.Count - 1].ChildRecords as RecordCollectionBase);
                        }
                        else 
                        {
                            //Recursively call again for the next level of groupbyrows
                            GroupByRecord groupByRecord = record as GroupByRecord;
                            ExportXamDataGridRecords(groupByRecord.ChildRecords);
                        }                        
                    }
                }
               }
            }

        private WorksheetRow MakeWorkSheetHeaderRow(Record record)
        {
            //get the level of hierarchy
            int level = GetHierarchichalLevel(record);
            //get the worksheetRow to work with
            WorksheetRow excelWorkSheetRow = excelWorksheet.Rows[excelWorksheetCounter];
            //set the outlineLevel for the excel row
            excelWorkSheetRow.OutlineLevel = level;
            if (level == 0)
            {
                //always make sure the first rows are visible
                excelWorkSheetRow.Hidden = false;
            }
            else
            {
                //See if the records parent record is a GroubByRecord or regular Record.  This will determine 
                //whether we use record.ParentRecord or record.ParentDataRecord.  "A GroupByRecord is not a DataRecord".
                //We can't use ParentRecord for both because the ParentRecord of a NonGroupByRow child record can be an
                //ExpandableFieldRecord which will give incorrect IsExpanded state.
                if (record.ParentRecord.GetType() == typeof(GroupByRecord))
                {
                    excelWorkSheetRow.Hidden = !record.ParentRecord.IsExpanded;
                }
                else
                {
                    excelWorkSheetRow.Hidden = !record.ParentDataRecord.IsExpanded;
                }

            }
            //loop through the fields
            for (int i = 0; i < record.FieldLayout.Fields.Count; i++)
            {
                //Make sure we dont include the Expandable Field
                if (record.FieldLayout.Fields[i].IsExpandableResolved != true)
                {
                    //Format these rows as header rows
                    excelWorkSheetRow = FormatHeaderCells(excelWorkSheetRow, level, i);
                    //fill in the cells with the field labels from the grid.
                    excelWorkSheetRow.Cells[i + level].Value = record.FieldLayout.Fields[i].Label.ToString();
                }
            }
            //increment the row counter;
            excelWorksheetCounter++;
            //return the excel WorksheetRow
            return excelWorkSheetRow;
        }

        private WorksheetRow FormatHeaderCells(WorksheetRow excelWorkSheetRow, int level, int i)
        {
            //Setting the fillPatern and color of the fill for the HeaderCells.
            excelWorkSheetRow.Cells[i + level].CellFormat.FillPatternForegroundColor = System.Drawing.Color.LightGray;
            excelWorkSheetRow.Cells[i + level].CellFormat.FillPattern = FillPatternStyle.Solid;
            return excelWorkSheetRow;
        }
    
        private WorksheetRow MakeWorkSheetRow(Record record)
        {
            int level = GetHierarchichalLevel(record);
            DataRecord dataRecord = record as DataRecord;
            GroupByRecord groupByRecord = record as GroupByRecord;

            //Create a WorkSheetRow for the Excel Worksheet we are working on based on a counter
            WorksheetRow excelWorkSheetRow = excelWorksheet.Rows[excelWorksheetCounter];

            if (record != null && level > 0)
            {
                //Set the OutlineLevel or the WorkSheetRow based on the nested depth of the DataRecord
                //Set the WorkSheetRow hidden it will appear collapsed or expanded in the WorkSheet
                //_based on the expanded state of the DataRecord

                if (record.ParentRecord.GetType() == typeof(GroupByRecord))
                {
                    excelWorkSheetRow.Hidden = !record.ParentRecord.IsExpanded;
                }
                else
                {
                    excelWorkSheetRow.Hidden = !record.ParentDataRecord.IsExpanded;
                }
                //Set the outline level of the row
                excelWorkSheetRow.OutlineLevel = level;
            }

            //Fill the Worksheet Cells with the values from the DataRecord Cells Subtracting 1 for the Relation 
            //_column if it is a normal DataRecord
            if (record.GetType() == typeof(DataRecord) && dataRecord != null)
            {
                for (int i = 0; i < dataRecord.Cells.Count; i++)
                {
                    if (dataRecord.Cells[i].Field.IsExpandableResolved != true)
                    {
                        excelWorkSheetRow.Cells[i + level].Value = dataRecord.Cells[i].Value;
                    }
                }
            }
            else //This is a GroupBy Record
            {
                //Set the Value to Description
                excelWorkSheetRow.Cells[0 + level].Value = groupByRecord.Description;
                for (int i = 0; i < groupByRecord.ChildRecords[0].FieldLayout.Fields.Count; i++)
                {
                    //Call Format Method
                    FormatGroupByRecordCells(excelWorkSheetRow, level, i);
                }
            }
            //Increment the counter so the next row can be added
            excelWorksheetCounter++;

            //Return the WorkSheetRow
            return excelWorkSheetRow;

        }

        private WorksheetRow FormatGroupByRecordCells(WorksheetRow excelWorkSheetRow, int level, int i)
        {
            //Style Points for the GroupByCells
            excelWorkSheetRow.Cells[i + level].CellFormat.BottomBorderColor = System.Drawing.Color.Black;
            excelWorkSheetRow.Cells[i + level].CellFormat.LeftBorderColor = System.Drawing.Color.LightBlue;
            excelWorkSheetRow.Cells[i + level].CellFormat.BottomBorderStyle = CellBorderLineStyle.Medium;
            excelWorkSheetRow.Cells[i + level].CellFormat.FillPatternForegroundColor = System.Drawing.Color.LightBlue;
            excelWorkSheetRow.Cells[i + level].CellFormat.FillPattern = FillPatternStyle.Solid;
            return excelWorkSheetRow;
        }

        private int GetHierarchichalLevel(Record record)
        {
            int level = 0;
            //Keep going until we have no parent record.  We need ParentRecord and GroupByRecord to handle
            //multiple levels of grouping.
            while (record.ParentDataRecord != null || record.ParentRecord != null)
            {
                //Check the type of the record of the parent so we know to look for ParentRecord or ParentDataRecord
                if (record.ParentRecord.GetType() == typeof(GroupByRecord))
                {
                    level++;
                    record = record.ParentRecord;
                }
                else
                {
                    level++;
                    record = record.ParentDataRecord as Record;
                }
            }
            return level;
        }




    }
}
